/*
 *	controller -- not much happens unless the controller requests it
 *
 * Copyright 1989 by W J Andrew Fyfe.  All rights reserved.  Permission to
 * copy this program is given provided that the copy is not sold and that
 * this copyright notice is included.
 */

#include "uw.h"

#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <sys/ipc.h>
#include <sys/msg.h>

static void startup_0(void);
static void new_window(int logical_window);
static int all_windows_gone(void);
static int stop = 0;

static void catch(void);

extern int errno, sys_nerr;
extern char *sys_errlist[];

static void
catch_alrm(void)
{
    signal(SIGALRM, catch_alrm);
    alarm(TIMEOUT);
}

void
controller(void)
{
    struct message msg;
    char buffer[128];

    startup_0();

    signal(SIGHUP, catch);
    signal(SIGALRM, catch_alrm);
    alarm(TIMEOUT);

    while(1) {
	if (stop || !modem_connected()) {
	    send_rd_wr(DIE_CMD, 0, 0);
	    break;
	}
	if (msgrcv(c_id, &msg, MSG_SIZE, MSG_TYPE, 0) < 0)
	    continue;
	alarm(0);
	if (msg.text[SERVER_BYTE] & P1_INTERNAL) {
	    switch(msg.text[SERVER_BYTE]) {
	    case P1_INT_DIE:
		send_rd_wr(DIE_CMD, 0, 0);
		return;
	    case P1_INT_CHDIR:
		if (msg.text[STRING_VALUE] != '\0' &&
		    chdir(&msg.text[STRING_VALUE]) < 0) {
			strcpy(buffer, "chdir failed: ");
			strcat(buffer, (errno >= sys_nerr || errno < 0) ?
				"Unknown error" : sys_errlist[errno]);
		}
		else if (getcwd(buffer, sizeof(buffer)) == NULL) {
		    strcpy(buffer, "getcwd failed: ");
		    strcat(buffer, (errno >= sys_nerr || errno < 0) ?
			    "Unknown error" : sys_errlist[errno]);
		}
		send_reader_string(MSG_CMD, buffer);
		break;
	    case P1_INT_UPLOAD:
		send_rd_wr(PAUSE_CMD, 0, 0);
		if (access(&msg.text[STRING_VALUE], R_OK) == 0) {
		    strcpy(buffer, upload(&msg.text[STRING_VALUE]));
		}
		else {
		    strcpy(buffer, &msg.text[STRING_VALUE]);
		    strcat(buffer, ": ");
		    strcat(buffer, (errno >= sys_nerr || errno < 0) ?
			    "Unknown error" : sys_errlist[errno]);
		}
		send_rd_wr(CONT_CMD, 0, 0);
		send_reader_string(MSG_CMD, buffer);
		break;
	    case P1_INT_DOWNLOAD:
		send_rd_wr(PAUSE_CMD, 0, 0);
		strcpy(buffer, download());
		send_rd_wr(CONT_CMD, 0, 0);
		send_reader_string(MSG_CMD, buffer);
		break;
	    }
	}
	else {
	    switch(msg.text[SERVER_BYTE] & P1_FN) {
	    case P1_FN_NEWW:
		if (msg.text[SERVER_BYTE] & P1_WINDOW)
		    new_window(msg.text[SERVER_BYTE] & P1_WINDOW);
		break;
	    case P1_FN_KILLW:
		if (msg.text[SERVER_BYTE] & P1_WINDOW) {
		    send_rd_wr(KILL_WINDOW_CMD, 0,
			msg.text[SERVER_BYTE] & P1_WINDOW);
		    close_window(msg.text[SERVER_BYTE] & P1_WINDOW);
		    if (all_windows_gone())
			startup_0();
		}
		break;
	    case P1_FN_MAINT:
		switch (msg.text[SERVER_BYTE] & P1_MF) {
		case P1_MF_ENTRY:
		    if (protocol == PROTOCOL_0)
			close_window(0);
		    else if (protocol == PROTOCOL_1)
			close_all_windows();
		    send_rd_wr(PROTOCOL_1_CMD, 0, 0);
		    protocol = PROTOCOL_1;
		    break;
		case P1_MF_EXIT:
		    if (protocol == PROTOCOL_1) {
			close_all_windows();
			startup_0();
		    }
		    break;
		}
		break;
	    }
	}
	alarm(TIMEOUT);
    }
    return;
}

static void
startup_0(void)
{
    int window, fd;

    protocol = PROTOCOL_0;

    fd = get_window("Unix Windows");
    window = window_number(fd);

    if (window < 0) {
	fprintf(stderr, "%s: can't open base window: ", cmd_name);
	perror("");
	send_rd_wr(DIE_CMD, 0, 0);
	stop = 1;
    }

    win_info[0].fd = fd;
    win_info[0].window_number = window;

    send_rd_wr(PROTOCOL_0_CMD, window, 0);
}

static void
catch(void)
{
    stop = 1;

    signal(SIGHUP, SIG_IGN);
}

static void
new_window(int logical_window)
{
    int window, fd;
    char title[80];

    sprintf(title, "UW -- window %d", logical_window);
    fd = get_window(title);
    window = window_number(fd);

    win_info[logical_window].fd = fd;
    win_info[logical_window].window_number = window;

    send_rd_wr(CREATE_WINDOW_CMD, window, logical_window);
}

static int
all_windows_gone(void)
{
    int i;

    for (i=1; i <= MAX_WINDOWS; ++i)
	if (win_info[i].fd >= 0)
	    return 0;
    return 1;
}
