/*
 *	msg_queues -- create and destroy the message queues
 *
 * Copyright 1989 by W J Andrew Fyfe.  All rights reserved.  Permission to
 * copy this program is given provided that the copy is not sold and that
 * this copyright notice is included.
 */
 
#include "uw.h"

#include <stdio.h>
#include <sys/ipc.h>
#include <sys/msg.h>

key_t c_key, r_key, w_key;
int c_id = -1, r_id = -1, w_id = -1;

/*
 * The 'c' queue is to send messages to the control process.
 * Both the reader and the write can do this.
 *
 * The 'r' queue is to send messages from the controller to the reader.
 *
 * The 'w' queue is to send messages from the controller to the writer.
 */
int
create_message_queues(void)
{
    c_key = ftok(UW_PROGRAM, 'c');
    if (c_key < 0) {
	fprintf(stderr, "%s: ftok failed (control): ", cmd_name);
	perror("");
	return -1;
    }
    c_id = msgget(c_key, IPC_CREAT | IPC_EXCL | 0722);
    if (c_id < 0) {
	fprintf(stderr, "%s: msgget failed (control): ", cmd_name);
	perror("");
	return -1;
    }

    r_key = ftok(UW_PROGRAM, 'r');
    if (r_key < 0) {
	fprintf(stderr, "%s: ftok failed (reader): ", cmd_name);
	perror("");
	return -1;
    }
    r_id = msgget(r_key, IPC_CREAT | IPC_EXCL | 0744);
    if (r_id < 0) {
	fprintf(stderr, "%s: msgget failed (reader): ", cmd_name);
	perror("");
	return -1;
    }

    w_key = ftok(UW_PROGRAM, 'w');
    if (w_key < 0) {
	fprintf(stderr, "%s: ftok failed (writer): ", cmd_name);
	perror("");
	return -1;
    }
    w_id = msgget(w_key, IPC_CREAT | IPC_EXCL | 0744);
    if (w_id < 0) {
	fprintf(stderr, "%s: msgget failed (writer): ", cmd_name);
	perror("");
	return -1;
    }

    return 0;
}

void
delete_message_queues(void)
{
    if (c_id >= 0)
	msgctl(c_id, IPC_RMID, (struct msqid_ds *)0);
    if (r_id >= 0)
	msgctl(r_id, IPC_RMID, (struct msqid_ds *)0);
    if (w_id >= 0)
	msgctl(w_id, IPC_RMID, (struct msqid_ds *)0);
}
