/*
 *	updnload -- do file trasfer between 3b1 and host
 *
 * Copyright 1989 by W J Andrew Fyfe.  All rights reserved.  Permission to
 * copy this program is given provided that the copy is not sold and that
 * this copyright notice is included.
 */
 
#include "uw.h"
#include "config.h"

#include <stdarg.h>
#include <string.h>
#include <fcntl.h>
#include <stdio.h>
#include <termio.h>
#include <sys/wait.h>

extern int errno, sys_nerr;
extern char *sys_errlist[];

static char *
updnload(const char *path, ...)
{
    va_list args;
    struct termio t;
    int pid, ret;
    union wait status;
    static char buffer[128];

    if (ioctl(phone_fd, TCGETA, &t) < 0) {
	sprintf(buffer, "%s: can't get termio for phone fd %d: ",
	    cmd_name, phone_fd);
	strcat(buffer, (errno >= sys_nerr || errno < 0) ?
		"Unknown error" : sys_errlist[errno]);
	return buffer;
    }

    pid = fork();
    if (pid < 0) {
	va_start(args, path);
	sprintf(buffer, "%s: can't fork \"%s\": ",
	    cmd_name, *(const char **)args);
	strcat(buffer, (errno >= sys_nerr || errno < 0) ?
		"Unknown error" : sys_errlist[errno]);
	va_end(args);
    }
    else if (pid == 0) {			/* child */
	setpgrp();
	close(0);
	close(1);
	if (fcntl(phone_fd, F_DUPFD, 0) < 0) {
	    fprintf(stderr, "%s: can't dup phone fd %d to 0: ",
		cmd_name, phone_fd);
	    perror("");
	    exit(1);
	}
	if (fcntl(phone_fd, F_DUPFD, 1) < 0) {
	    fprintf(stderr, "%s: can't dup phone fd %d to 1: ",
		cmd_name, phone_fd);
	    perror("");
	    exit(1);
	}
	setgid(getgid());
	setuid(getuid());
	va_start(args, path);
	execv(path, (const char **)args);
	fprintf(stderr, "%s: can't exec \"%s\": ", cmd_name, path);
	perror("");
	va_end(args);
	exit(1);
    }
    else {					/* parent */
	va_start(args, path);
	while (1) {
	    ret = wait(&status);
	    if (ret < 0) {
		fprintf(stderr, "\n%s: %s: wait failed: ",
		    cmd_name, *(const char **)args);
		perror("");
	    }
	    else if (ret != pid) {
		fprintf(stderr,
		    "\n%s: %s: "
		    "wait returned status [%.8x] for unknown pid %d\n",
		    cmd_name, *(const char **)args, status.w_status, ret);
	    }
	    else
		break;
	}
	if (status.w_termsig == 0) {
	    if (status.w_retcode != 0)
		sprintf(buffer, "%s: exit status %d",
		    *(const char **)args, status.w_retcode);
	    else
		strcpy(buffer, "up/down load complete");
	}
	else
	    sprintf(buffer, "%s: killed signal %d",
		*(const char **)args, status.w_termsig);
	fprintf(stderr, "\n%s: %s\n", cmd_name, buffer);
	va_end(args);
    }

    if (ioctl(phone_fd, TCSETA, &t) < 0) {
	fprintf(stderr, "%s: can't set termio for phone fd %d: ",
	    cmd_name, phone_fd);
	perror("");
    }

    return buffer;
}

char *
upload(const unsigned char *name)
{
    return updnload(UPLOAD_PROGRAM, UPLOAD_NAME, name, (char *)0);
}

char *
download(void)
{
    return updnload(DOWNLOAD_PROGRAM, DOWNLOAD_NAME, (char *)0);
}
