/*
 *	utils -- open and close windows, etc.
 *
 * Copyright 1989 by W J Andrew Fyfe.  All rights reserved.  Permission to
 * copy this program is given provided that the copy is not sold and that
 * this copyright notice is included.
 */
 
#include "uw.h"
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <termio.h>
#include <signal.h>
#include <sys/ipc.h>
#include <sys/msg.h>

static int set_window_modes(int fd);

void
close_window(int logical_window)
{
    if (win_info[logical_window].fd >= 0) {
	close(win_info[logical_window].fd);
	win_info[logical_window].fd = -1;
	win_info[logical_window].window_number = -1;
    }
}

void
close_all_windows(void)
{
    int i;

    for (i=1; i<=MAX_WINDOWS; ++i)
	close_window(i);
}

int
open_window(int logical_window, int real_window, int rdonly)
{
    char window_name[32];

    if (real_window <= 0)
	return -1;

    if (win_info[logical_window].fd != -1)
	return -1;

    sprintf(window_name, WINDOW_DEVICE_BY_NUMBER, real_window);
    win_info[logical_window].fd =
	open(window_name, (rdonly == READ_ONLY ? O_RDONLY : O_WRONLY), 0);

    if (win_info[logical_window].fd < 0) {
	fprintf(stderr, "%s: can't open %s: ", cmd_name, window_name);
	perror("");
	return -1;;
    }
    if (set_window_modes(win_info[logical_window].fd) < 0)
	return -1;

    win_info[logical_window].window_number = real_window;

    return 0;
}

static int
set_window_modes(int fd)
{
    struct termio t;

    if (ioctl(fd, TCGETA, &t) < 0) {
	fprintf(stderr, "%s: can't get termio for fd %d: ", cmd_name, fd);
	perror("");
	return -1;
    }

    t.c_cc[VMIN] = 1;
    t.c_cc[VTIME] = 0;
    t.c_iflag = 0;
    t.c_oflag = 0;
    t.c_lflag = 0;

    if (ioctl(fd, TCSETA, &t) < 0) {
	fprintf(stderr, "%s: can't set termio for fd %d: ", cmd_name, fd);
	perror("");
	return -1;
    }

    return 0;
}

void
send_controller(int byte)
{
    struct message msg;

    msg.type = MSG_TYPE;
    msg.text[COMMAND] = SERVER_CMD;
    msg.text[SERVER_BYTE] = byte;
    msgsnd(c_id, &msg, MSG_SIZE, IPC_NOWAIT);
}

void
send_controller_string(int byte, char str[WTXTLEN])
{
    struct message msg;

    msg.type = MSG_TYPE;
    msg.text[COMMAND] = SERVER_CMD;
    msg.text[SERVER_BYTE] = byte;
    strncpy(&msg.text[STRING_VALUE], str, WTXTLEN);
    msgsnd(c_id, &msg, MSG_SIZE, IPC_NOWAIT);
}

void
send_rd_wr(int command, int param1, int param2)
{
    struct message msg;
    static int first = 1;

    msg.type = MSG_TYPE;
    msg.text[COMMAND] = command;
    msg.text[PARAM1] = param1;
    msg.text[PARAM2] = param2;
    msgsnd(r_id, &msg, MSG_SIZE, IPC_NOWAIT);
    msgsnd(w_id, &msg, MSG_SIZE, IPC_NOWAIT);
    if (first)			/* no signals on the first message */
	first = 0;
    else {
	kill(pid_rd, SIGUSR1);
	kill(pid_wr, SIGUSR1);
    }
}

void
send_reader_string(int command, char str[WTXTLEN])
{
    struct message msg;

    msg.type = MSG_TYPE;
    msg.text[COMMAND] = command;
    strncpy(&msg.text[STRING_VALUE], str, WTXTLEN);
    msgsnd(r_id, &msg, MSG_SIZE, IPC_NOWAIT);
}
