/*
 *	uw.h -- random stuff needed by the C source
 *
 * Copyright 1989 by W J Andrew Fyfe.  All rights reserved.  Permission to
 * copy this program is given provided that the copy is not sold and that
 * this copyright notice is included.
 */
 
#include <stdlib.h>
#include <sys/types.h>
#include <sys/window.h>
#include "uw_pcl.h"

#define WINDOW_DEVICE		"/dev/window"
#define WINDOW_DEVICE_BY_NUMBER "/dev/w%d"

#define UW_PROGRAM		"/usr/local/bin/uw"

#define STDIN			0

#define MAX_WINDOWS		P1_NWINDOW

#define READ_ONLY		1
#define WRITE_ONLY		0

#define PROTOCOL_NONE		-1
#define PROTOCOL_0		1
#define PROTOCOL_1		2

#define PROTOCOL_0_CMD		'0'
#define PROTOCOL_1_CMD		'1'
#define CREATE_WINDOW_CMD	'C'
#define KILL_WINDOW_CMD		'K'
#define DIE_CMD			'D'
#define SERVER_CMD		'S'
#define PAUSE_CMD		'P'
#define CONT_CMD		'R'
#define MSG_CMD			'M'

#define COMMAND			0
#define PARAM1			1
#define PARAM2			2
#define REAL_WINDOW		PARAM1
#define LOGICAL_WINDOW		PARAM2
#define SERVER_BYTE		PARAM1
#define STRING_VALUE		PARAM2

#define MSG_SIZE		(WTXTLEN + 2)
#define MSG_TYPE		1

#define IAC			P1_IAC
#define XON			'\021'
#define XOFF			'\023'
#define META_BIT		0200

#define TIMEOUT			5

struct message {
    long type;
    unsigned char text[MSG_SIZE];
};

struct window_info {
    int window_number;
    int fd;
};

extern char *cmd_name;
extern int phone_fd;
extern struct window_info win_info[MAX_WINDOWS + 1];
extern int pid_rd, pid_wr;
extern int protocol;
extern key_t c_key, r_key, w_key;
extern int c_id, r_id, w_id;
extern int use_meta, kbd_esc_char;

int dial_number(const char *line, const char *speed, const char *number);
void undial_number(const char *line);
int modem_connected(void);

int create_message_queues(void);
void delete_message_queues(void);

int init_window(void);
int get_window(const char *win_name);
int window_number(int fd);

void close_window(int logical_window);
void close_all_windows(void);
int open_window(int logical_window, int real_window, int rdonly);

void send_controller(int byte);
void send_controller_string(int byte, char str[WTXTLEN]);
void send_rd_wr(int command, int param1, int param2);
void send_reader_string(int command, char str[WTXTLEN]);

volatile void reader(void);
volatile void writer(void);
void controller(void);

char *upload(const unsigned char *);
char *download(void);

int msgctl(int, int, void *);
int msgget(key_t, int);
int msgsnd(int, void *, int, int);
int msgrcv(int, void *, int, long, int);
