/*
 *	window -- create windows
 *
 * Copyright 1989 by W J Andrew Fyfe.  All rights reserved.  Permission to
 * copy this program is given provided that the copy is not sold and that
 * this copyright notice is included.
 */
 
#include "uw.h"

#include <stdio.h>
#include <string.h>
#include <sys/window.h>
#include <fcntl.h>
#include <termio.h>
#include <sys/stat.h>
#include <sys/sysmacros.h>

static int window_major_number;

int
init_window(void)
{
    struct stat buf;
    int stat(const char *, struct stat *);

    if (stat(WINDOW_DEVICE, &buf) < 0) {
	fprintf(stderr, "%s: can't stat %s: ", cmd_name, WINDOW_DEVICE);
	perror("");
	return -1;
    }

    window_major_number = major(buf.st_rdev);
    return 0;
}

int
get_window(const char *win_name)
{
    struct uwdata d;
    struct utdata t;
    int fd;

    /*
     * Create a new window.
     */
    fd = open(WINDOW_DEVICE, O_WRONLY, 0);
    if (fd < 0) {
	fprintf(stderr, "%s: can't open %s: ", cmd_name, WINDOW_DEVICE);
	perror("");
	return -1;
    }

    /*
     * Set the window to use the full screen
     */
    d.uw_x = 0;
    d.uw_y = WLINE(2);
    d.uw_width = WINWIDTH;
    d.uw_height = WINHEIGHT - d.uw_y;
    d.uw_uflags = NBORDER;
    if (ioctl(fd, WIOCSETD, &d) < 0) {
	fprintf(stderr, "%s: window ioctl failed: ", cmd_name);
	perror("");
	close(fd);
	return -1;
    }

    /*
     * Give the window a name.  We don't care if it fails.
     */
    t.ut_num = WTXTUSER;
    strncpy(t.ut_text, win_name, WTXTLEN);
    t.ut_text[WTXTLEN-1] = '\0';
    (void)ioctl(fd, WIOCSETTEXT, &t);

    (void)ioctl(fd, WIOCSELECT, (char *)0);

    return fd;
}

int
window_number(int fd)
{
    struct stat buf;
    int fstat(int, struct stat *);

    if (fd < 0)
	return -1;

    if (fstat(fd, &buf) < 0) {
	fprintf(stderr, "%s: can't stat fd %d: ", cmd_name, fd);
	perror("");
	return -1;
    }

    if (major(buf.st_rdev) == window_major_number)
	return minor(buf.st_rdev);
    else
	return -1;
}
